@@ -137,6 +137,7 @@ sub new {
 =item B<run>
 
 Runner itself, service sub
+Never returns. Insted exit()s with error or success, or throws exception in some cases.
 
 =back
 
@@ -146,18 +147,18 @@ sub run {
     my $self = shift;
     unless ($ARGV[0]) {
         $self->usage();
-        return 1;
+        exit(0);
     }
 
     my $sub = $ARGV[0];
     $sub =~ s/-/_/g;
     if ($self->can($sub)) {
-        $self->$sub();
+        $self->$sub() ? exit(0) : exit(1);
     }
     else {
         $self->usage();
+        exit(1);
     }
-    return 1;
 }
 
 sub start {
@@ -249,6 +250,13 @@ sub stop_old {
     shift->_stop('old')
 }
 
+=item restart
+
+stops and start daemon, stop errors (except fatals) are ignored. start return code
+returned as restart return code
+
+=cut
+
 sub restart {
     my $self = shift;
 
@@ -258,8 +266,7 @@ sub restart {
         Time::HiRes::usleep(1000);
     }
 
-    $self->start();
-    return 1;
+    return $self->start();
 }
 
 # same as start, but
@@ -324,7 +331,10 @@ sub _reload {
         Time::HiRes::usleep(10_000) while (! -f $self->{_args}->{progress_log});
         print "Log below:\n";
         system('tail', '-f', $self->{_args}->{progress_log});
+        return; # should unreachable
     }
+
+    return 1;
 }
 
 sub reload {
@@ -340,6 +350,7 @@ sub status {
 
     if (!$self->{pid}) {
         print DAEMON_IS_NOT_RUNNING;
+        return;
     }
     elsif ($self->is_alive()) {
         if ($self->new_pid_is_alive()) {
@@ -348,9 +359,8 @@ sub status {
         else {
             print DAEMON_ALREADY_RUNNING;
         }
+        return 1;
     }
-
-    exit 0;
 }
 
 
@@ -365,7 +375,7 @@ sub info {
             $p->pid(), $p->command(), $p->user(), $self->{_args}->{progress_log} // 'NONE';
     }
 
-    exit 0;
+    return 1;
 
 }
 
